import tensorflow as tf
from scipy import signal
import numpy as np
import math


def wf1_loss(y_hat, y, beta=1.0):
    #y_hat: the predicted segmentation, a continuous valued tensor of size: batch_size x height x width x 1
    #y: the ground truth, a tensor of floats that take value 1.0 or 0.0 size: batch_size x height x width x 1
    #beta: a float scalar, controls the tradeoff between precision and recall (p)

    

    k_width = 19
    k = signal.gaussian(k_width,2.236)
    k = np.outer(k,k)
    k = np.reshape(k,(k_width,k_width,1,1))

    for ii in range(k_width):
      for jj in range(k_width):
        it = ii-(k_width/2.0-1.0)
        jt = ii-(k_width/2.0-1.0)

        k[ii,jj,0,0] = (1.0/np.sqrt(2*math.pi*2.263*2.263))*np.exp(-(it*it+jt*jt)/(2*2.263*2.263))
        

    k_width2 = 31
    k2 = signal.gaussian(k_width2,2.236)
    k2 = k2/np.max(k2)
    k2 = np.outer(k2,k2)
    k2 = np.reshape(k2,(k_width2,k_width2,1,1))

    k_max =np.zeros((k_width2,k_width2,1,k_width2*k_width2))
    ii_jj=0
    for ii in range(k_width2):
      for jj in range(k_width2):
        it = ii-(k_width2/2.0-1.0)
        jt = jj-(k_width2/2.0-1.0)

        k_max[ii,jj,0,ii_jj] = np.exp(np.log(0.5)/5.0*np.sqrt(it*it+jt*jt))
        ii_jj += 1


    k_ =     tf.constant(k,     dtype=tf.float32)
    k_max_ = tf.constant(k_max, dtype=tf.float32)

    B_ = 2.0 - tf.reduce_max(tf.nn.conv2d(y,k_max_,[1,1,1,1],"SAME"), reduction_indices=3, keep_dims=True)

    E = (y-y_hat)*(y-y_hat)
    #E = np.absolute(y-y_hat)
    #E = tf.Print(E_fake,[E_fake])

    EA = (1.0-y)*E+y*tf.nn.conv2d(y*E,k_,[1,1,1,1],"SAME")
    E_w = tf.minimum(E,EA)*B_

    TP = tf.reduce_sum((1.0-E_w)*y,reduction_indices=[1,2])
    #TP = tf.Print(TP_fake,[TP_fake])
    TN = tf.reduce_sum((1.0-E_w)*(1.0-y),reduction_indices=[1,2])
    FN = tf.reduce_sum(E_w*y,reduction_indices=[1,2])
    FP = tf.reduce_sum(E_w*(1.0-y),reduction_indices=[1,2])
    P = (TP)/(TP+FP+1.0)
    R = (TP)/(TP+FN+1.0)
    loss_func = tf.reduce_mean(1.0 - (1.0+beta*beta)*P*R/(beta*beta*P+R+1e-10)) # 1- because we are minimizing the quantity we wish to maximize

    return loss_func
  